@@ -2,12 +2,14 @@
 
 =head1 NAME
 
+(This is still just a stub.  See also FlatFile::DataStore::FMTEYEWTK.)
+
 FlatFile::DataStore::Tutorial - POD containing in-depth discussion
 of and tutorials for using FlatFile::DataStore.
 
 =head1 VERSION
 
-Discusses FlatFile::DataStore version 0.07.
+Discusses FlatFile::DataStore version 0.11.
 
 =head1 SYNOPSYS
 
@@ -72,13 +74,14 @@ have a pointer to a "next" version, because there isn't one.
 Each record is stored with a I<preamble>, which is a fixed-length
 string of fields containing:
 
- - crud indicator       (flag for created, updated, deleted, etc.)
- - transaction number   (incremented when a record is touched)
- - date                 (of the "transaction")
- - key number           (record sequence number)
- - record length        (in bytes)
- - user data            (for out-of-band* user-defined data)
- - "this" file number   (linked list pointers ...)
+ - crud indicator        (flag for created, updated, deleted, etc.)
+ - transaction indicator (flag for created, updated, deleted, etc.)
+ - transaction number    (incremented when a record is touched)
+ - date                  (of the "transaction")
+ - key number            (record sequence number)
+ - record length         (in bytes)
+ - user data             (for out-of-band* user-defined data)
+ - "this" file number    (linked list pointers ...)
  - "this" seek position
  - "prev" file number
  - "prev" seek position
